@@ -7,8 +7,8 @@
 //! It adds greater safety and type correctness over the underlying C library
 //! API, while trying to remain very close to it.
 //!
-//! It offers the choice of using it [**more like Rust**](#like-rust)
-//! and/or [**more like C**](#like-C).
+//! It offers the choice of using it [*more like Rust*](#like-rust)
+//! and/or [*more like C*](#like-C).
 //!
 //! ## Like Rust
 //!
@@ -16,22 +16,9 @@
 //! constructors, and error handling with the `NcResult` enum:
 //!
 //! ### Example
+#![doc = concat!["```\n", include_str!("../examples/hello-world-rust.rs"), "\n```" ]]
 //!
-//! ```rust
-//! use libnotcurses_sys::*;
-//!
-//! # #[cfg(not(miri))]
-//! fn main() -> NcResult<()> {
-//!     let nc = Nc::new_cli()?;
-//!     let plane = nc.stdplane();
-//!     plane.putstr("hello world")?;
-//!     nc.render()?;
-//!     nc.stop()?;
-//!     Ok(())
-//! }
-//! # #[cfg(miri)]
-//! # fn main() {}
-//! ```
+//! ### Notes on the Rust API
 //!
 //! The `Drop` trait is not implemented for any wrapping type in this library.
 //!
@@ -48,6 +35,9 @@
 //! leverage type checking, and they implement methods through traits
 //! (e.g. `NcChannelApi` must be in scope to use the `NcChannel` methods.
 //!
+//! Several methods are declared unsafe when they have addittional contracts to
+//! manually upheld in order to avoid *UB*.
+//!
 //! ### even more like Rust
 //!
 //! The *WIP* sister crate
@@ -60,56 +50,13 @@
 //! You can access the imported, or reimplemented C API functions directly,
 //! and use it in a very similar way as the C library is used.
 //!
-//! It requires the use of unsafe, since most functions are wrapped directly
-//! by `bindgen` marked as such.
+//! It requires more use of unsafe, since it has less safer abstractions.
 //!
 //! Error handling is done this way by checking the returned `NcIntResult`,
 //! or in case of receiving a pointer, by comparing it to `null_mut()`.
 //!
 //! ### Example
-//!
-//! ```rust
-//! use core::ptr::{null, null_mut};
-//! use std::process::exit;
-//!
-//! use libnotcurses_sys::{c_api::*};
-//!
-//! # #[cfg(not(miri))]
-//! fn main() {
-//!     let options = ffi::notcurses_options {
-//!         termtype: null(),
-//!         renderfp: null_mut(),
-//!         loglevel: 0,
-//!         margin_t: 0,
-//!         margin_r: 0,
-//!         margin_b: 0,
-//!         margin_l: 0,
-//!         flags: NCOPTION_NO_ALTERNATE_SCREEN
-//!         | NCOPTION_PRESERVE_CURSOR
-//!         | NCOPTION_SUPPRESS_BANNERS
-//!     };
-//!     unsafe {
-//!         let nc = notcurses_init(&options, null_mut());
-//!         if nc.is_null() {
-//!             exit(1);
-//!         }
-//!         let plane = notcurses_stdplane(nc);
-//!         let cols = ncplane_putstr(&mut *plane, "hello world");
-//!         if cols < NCRESULT_OK {
-//!             notcurses_stop(nc);
-//!             exit(cols.abs());
-//!         }
-//!         if notcurses_render(nc) < NCRESULT_OK {
-//!             exit(2);
-//!         }
-//!         if notcurses_stop(nc) < NCRESULT_OK {
-//!             exit(3);
-//!         }
-//!     }
-//! }
-//! # #[cfg(miri)]
-//! # fn main() {}
-//! ```
+#![doc = concat!["```\n", include_str!("../examples/hello-world-c.rs"), "\n```" ]]
 //!
 //! ### The `notcurses` C API docs
 //!
@@ -167,7 +114,7 @@ pub mod widgets;
 // in which case are both aliases over `u32`.
 pub use align::{NcAlign, NcAlignApi};
 pub use alpha::{NcAlpha, NcAlphaApi};
-pub use blitter::{NcBlitter, NcBlitterApi, NcBlitterGeometry};
+pub use blitter::{NcBlitter, NcBlitterApi};
 pub use capabilities::NcCapabilities;
 pub use cell::NcCell;
 pub use channel::{NcChannel, NcChannelApi, NcChannels, NcChannelsApi, NcComponent, NcRgb};
@@ -177,14 +124,14 @@ pub use error::{NcError, NcIntResult, NcIntResultApi, NcResult};
 pub use fade::{NcFadeCb, NcFadeCtx};
 pub use fd::{NcFdPlane, NcFdPlaneOptions, NcSubproc, NcSubprocOptions};
 pub use file::NcFile;
-pub use input::{NcEvType, NcEvTypeApi, NcInput};
+pub use input::{NcEvType, NcEvTypeApi, NcInput, NcMiceEvents, NcMiceEventsApi, NcReceived};
 pub use key::NcKey;
 pub use log_level::{NcLogLevel, NcLogLevelApi};
 pub use macros::*;
 pub use notcurses::{Nc, NcOptions};
 pub use palette::{NcPalette, NcPaletteIndex};
-pub use pixel::{NcPixel, NcPixelGeometry, NcPixelImpl, NcPixelImplApi};
-pub use plane::{NcPlane, NcPlaneOptions};
+pub use pixel::{NcPixel, NcPixelApi, NcPixelGeometry, NcPixelImpl, NcPixelImplApi};
+pub use plane::{NcPlane, NcPlaneOptions, NcPlaneOptionsBuilder};
 pub use r#box::{NcBoxMask, NcBoxMaskApi};
 pub use resizecb::{NcResizeCb, NcResizeCbApi, NcResizeCbUnsafe};
 pub use scale::{NcScale, NcScaleApi};
@@ -192,7 +139,9 @@ pub use stats::NcStats;
 pub use string::NcString;
 pub use style::{NcStyle, NcStyleApi};
 pub use time::NcTime;
-pub use visual::{NcRgba, NcVGeom, NcVisual, NcVisualOptions}; // ←
+pub use visual::{
+    NcRgba, NcVGeom, NcVisual, NcVisualGeometry, NcVisualOptions, NcVisualOptionsBuilder,
+};
 
 pub mod c_api {
     //! The C API including global constants, functions and structs.
@@ -222,6 +171,7 @@ pub mod c_api {
     pub use crate::channel::reimplemented::*;
     pub use crate::direct::reimplemented::*;
     pub use crate::input::reimplemented::*;
+    pub use crate::key::reimplemented::*;
     pub use crate::metric::reimplemented::*;
     pub use crate::notcurses::reimplemented::*;
     pub use crate::palette::reimplemented::*;
@@ -236,18 +186,18 @@ pub mod c_api {
     pub use crate::channel::constants::*;
     pub use crate::direct::constants::*;
     pub use crate::error::constants::*;
-    pub use crate::input::constants::*; // evdev
+    pub use crate::input::constants::*;
     pub use crate::key::constants::*;
     pub use crate::log_level::constants::*;
     pub use crate::metric::constants::*;
     pub use crate::notcurses::constants::*;
     pub use crate::palette::constants::*;
     pub use crate::pixel::constants::*;
-    pub use crate::plane::constants::*;
+    pub use crate::plane::options::constants::*;
     pub use crate::r#box::constants::*;
     pub use crate::scale::constants::*;
     pub use crate::style::constants::*;
-    pub use crate::visual::constants::*;
+    pub use crate::visual::options::constants::*;
     pub use crate::widgets::menu::constants::*;
     pub use crate::widgets::plot::constants::*;
     pub use crate::widgets::progbar::constants::*;
@@ -264,3 +214,35 @@ pub mod c_api {
     #[allow(unused_imports)]
     pub(crate) use helpers::*;
 }
+
+/// The library `examples/`.
+///
+/// Here you can see and navigate the examples as individual modules,
+/// as well as the common utility objects from [`utils`][examples::utils].
+///
+// Note that `cargo doc` doesn't detect changes made inside the `/examples/`
+// directory, unless something about the `examples` module has been modified,
+// (This maybe a side effect of using the [module `path` attribute][0])
+//
+// [0]: https://doc.rust-lang.org/reference/items/modules.html#the-path-attribute
+#[path = "../examples"]
+#[cfg(any(doc, test))]
+pub mod examples {
+    #![allow(dead_code)]
+
+    pub mod utils;
+    pub use utils::{Canvas, Counter};
+
+    // individual examples
+
+    #[path = "erase_region.rs"]
+    pub mod example_erase_region;
+    #[path = "info.rs"]
+    pub mod example_info;
+    #[path = "input.rs"]
+    pub mod example_input;
+    #[path = "pixel-cell.rs"]
+    pub mod example_pixel_cell;
+    #[path = "planes.rs"]
+    pub mod example_planes;
+}
