@@ -4,9 +4,10 @@ use core::ptr::{null, null_mut};
 
 use crate::{
     c_api::{self, notcurses_init},
-    cstring, error, error_ref_mut, rstring, rstring_free, Nc, NcAlign, NcBlitter, NcChannels,
-    NcDim, NcError, NcFile, NcInput, NcLogLevel, NcOptions, NcPixelImpl, NcPlane, NcResult,
-    NcScale, NcStats, NcStyle, NcStyleApi, NcTime,
+    cstring, error, error_ref_mut, rstring, rstring_free, Nc, NcAlign, NcBlitter, NcBlitterApi,
+    NcCapabilities, NcChannels, NcDim, NcError, NcFile, NcInput, NcLogLevel, NcMiceEvents,
+    NcMiceEventsApi, NcOptions, NcPixelImpl, NcPlane, NcReceived, NcResult, NcScale, NcStats,
+    NcStyle, NcStyleApi, NcTime, NcVGeom, NcVisual, NcVisualGeometry, NcVisualOptions,
 };
 
 /// # `NcOptions` Constructors
@@ -60,22 +61,26 @@ impl NcOptions {
         Self {
             termtype: null(),
             loglevel,
-            renderfp: null_mut(),
-            margin_t: margin_t as i32,
-            margin_r: margin_r as i32,
-            margin_b: margin_b as i32,
-            margin_l: margin_l as i32,
+            margin_t,
+            margin_r,
+            margin_b,
+            margin_l,
             flags,
         }
     }
 }
 
-/// # `Nc` Constructors
+/// # `Nc` Constructors and destructors
 //
 // TODO: rethink constructors
 impl Nc {
     /// New notcurses context (without banners).
-    pub fn new<'a>() -> NcResult<&'a mut Nc> {
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn new<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(NcOptions::SUPPRESS_BANNERS)
     }
 
@@ -86,7 +91,12 @@ impl Nc {
     /// - [`NcOptions::NO_ALTERNATE_SCREEN`][NcOptions#associatedconstant.NO_ALTERNATE_SCREEN]
     /// - [`NcOptions::NO_CLEAR_BITMAPS`][NcOptions#associatedconstant.NO_CLEAR_BITMAPS]
     /// - [`NcOptions::PRESERVE_CURSOR`][NcOptions#associatedconstant.PRESERVE_CURSOR]
-    pub fn new_cli<'a>() -> NcResult<&'a mut Nc> {
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn new_cli<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(
             NcOptions::SUPPRESS_BANNERS
                 | NcOptions::NO_ALTERNATE_SCREEN
@@ -98,32 +108,63 @@ impl Nc {
     /// New notcurses context, with banners.
     ///
     /// This is the default in the C library.
-    pub fn with_banners<'a>() -> NcResult<&'a mut Nc> {
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn with_banners<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(0)
     }
 
     /// New notcurses context, without an alternate screen (nor banners).
+    ///
     #[deprecated]
     #[doc(hidden)]
-    pub fn without_altscreen<'a>() -> NcResult<&'a mut Nc> {
+    pub unsafe fn without_altscreen<'a>() -> NcResult<&'a mut Nc> {
         Self::with_flags(NcOptions::NO_ALTERNATE_SCREEN | NcOptions::SUPPRESS_BANNERS)
     }
 
     /// New notcurses context, expects `NcOptions::*` flags.
-    pub fn with_flags<'a>(flags: u64) -> NcResult<&'a mut Nc> {
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn with_flags<'a>(flags: u64) -> NcResult<&'a mut Nc> {
         Self::with_options(NcOptions::with_flags(flags))
     }
 
     /// New notcurses context, expects [NcOptions].
-    pub fn with_options<'a>(options: NcOptions) -> NcResult<&'a mut Nc> {
-        let res = unsafe { notcurses_init(&options, null_mut()) };
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn with_options<'a>(options: NcOptions) -> NcResult<&'a mut Nc> {
+        let res = notcurses_init(&options, null_mut());
         error_ref_mut![res, "Nc.with_options()"]
     }
 
     /// New notcurses context, expects [NcLogLevel] and flags.
-    pub fn with_debug<'a>(loglevel: NcLogLevel, flags: u64) -> NcResult<&'a mut Nc> {
+    ///
+    /// # Safety
+    /// You must not create multiple `Nc` instances at the same time, on
+    /// the same thread. You must [`stop`][Nc#method.stop] the current one
+    /// before creating a new one.
+    pub unsafe fn with_debug<'a>(loglevel: NcLogLevel, flags: u64) -> NcResult<&'a mut Nc> {
         Self::with_options(NcOptions::with_all_options(loglevel, 0, 0, 0, 0, flags))
     }
+
+    /// Destroys the notcurses context.
+    ///
+    /// # Safety
+    /// You must not call this method repeatedly on the same `Nc` instance.
+    ///
+    /// *C style function: [notcurses_stop()][c_api::notcurses_stop].*
+    pub unsafe fn stop(&mut self) -> NcResult<()> {
+        error![c_api::notcurses_stop(self)]
+    }
 }
 
 /// # `Nc` methods
@@ -142,7 +183,7 @@ impl Nc {
         error![c_api::notcurses_align(availcols, align, cols)]
     }
 
-    /// Retrieves the current contents of the specified [NcCell][crate::NcCell]
+    /// Retrieves the current contents of the specified [`NcCell`][crate::NcCell]
     /// as last rendered, returning the `EGC` (or None on error) and writing
     /// out the [`NcStyle`] and the [`NcChannels`].
     ///
@@ -154,19 +195,18 @@ impl Nc {
         stylemask: &mut NcStyle,
         channels: &mut NcChannels,
     ) -> Option<String> {
-        let egc = unsafe { c_api::notcurses_at_yx(self, x as i32, y as i32, stylemask, channels) };
+        let egc = unsafe { c_api::notcurses_at_yx(self, y, x, stylemask, channels) };
         if egc.is_null() {
             return None;
         }
         Some(rstring_free![egc])
     }
 
-    /// Returns the bottommost [`NcPlane`] on the standard pile,
-    /// of which there is always at least one.
+    /// Returns [`NcCapabilities`].
     ///
-    /// *C style function: [notcurses_bottom()][c_api::notcurses_bottom].*
-    pub fn bottom(&mut self) -> &mut NcPlane {
-        unsafe { &mut *c_api::notcurses_bottom(self) }
+    /// *C style function: [notcurses_capabilities()][c_api::notcurses_capabilities].*
+    pub fn capabilities(&self) -> NcCapabilities {
+        unsafe { *c_api::notcurses_capabilities(self) }
     }
 
     /// Returns true if we can reliably use Unicode Braille.
@@ -175,7 +215,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_canbraille()][c_api::notcurses_canbraille].*
     pub fn canbraille(&self) -> bool {
-        unsafe { c_api::notcurses_canbraille(self) }
+        c_api::notcurses_canbraille(self)
     }
 
     /// Returns true if it's possible to set the "hardware" palette.
@@ -184,7 +224,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_canchangecolor()][c_api::notcurses_canchangecolor].*
     pub fn canchangecolor(&self) -> bool {
-        unsafe { c_api::notcurses_canchangecolor(self) }
+        c_api::notcurses_canchangecolor(self)
     }
 
     /// Returns true if fading is possible.
@@ -193,7 +233,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_canfade()][c_api::notcurses_canfade].*
     pub fn canfade(&self) -> bool {
-        unsafe { c_api::notcurses_canfade(self) }
+        c_api::notcurses_canfade(self)
     }
 
     /// Returns true if we can reliably use Unicode half blocks.
@@ -202,7 +242,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_canhalfblock()][c_api::notcurses_canhalfblock].*
     pub fn canhalfblock(&self) -> bool {
-        unsafe { c_api::notcurses_canhalfblock(self) }
+        c_api::notcurses_canhalfblock(self)
     }
 
     /// Returns true if loading images is possible.
@@ -223,13 +263,22 @@ impl Nc {
         unsafe { c_api::notcurses_canopen_videos(self) }
     }
 
+    /// Returns true if we can blit pixel-accurate bitmaps.
+    ///
+    /// See also [`check_pixel_support`][Nc#method.check_pixel_support].
+    ///
+    /// *C style function: [notcurses_canpixel()][c_api::notcurses_canpixel].*
+    pub fn canpixel(&self) -> bool {
+        c_api::notcurses_canpixel(self)
+    }
+
     /// Returns true if we can reliably use Unicode quadrant blocks.
     ///
     /// See also [`NcBlitter::BLIT_2x2`][NcBlitter#associatedconstant.NCBLIT_2x2].
     ///
     /// *C style function: [notcurses_canquadrant()][c_api::notcurses_canquadrant].*
     pub fn canquadrant(&self) -> bool {
-        unsafe { c_api::notcurses_canquadrant(self) }
+        c_api::notcurses_canquadrant(self)
     }
 
     /// Returns true if we can reliably use Unicode 13 sextants.
@@ -238,7 +287,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_cansextant()][c_api::notcurses_cansextant].*
     pub fn cansextant(&self) -> bool {
-        unsafe { c_api::notcurses_cansextant(self) }
+        c_api::notcurses_cansextant(self)
     }
 
     /// Returns true if it's possible to directly specify RGB values per cell,
@@ -246,7 +295,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_cantruecolor()][c_api::notcurses_cantruecolor].*
     pub fn cantruecolor(&self) -> bool {
-        unsafe { c_api::notcurses_cantruecolor(self) }
+        c_api::notcurses_cantruecolor(self)
     }
 
     /// Returns true if the encoding is UTF-8.
@@ -255,7 +304,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_canutf8()][c_api::notcurses_canutf8].*
     pub fn canutf8(&self) -> bool {
-        unsafe { c_api::notcurses_canutf8(self) }
+        c_api::notcurses_canutf8(self)
     }
 
     /// Checks for pixel support.
@@ -346,6 +395,13 @@ impl Nc {
         rstring_free![c_api::notcurses_detected_terminal(self)]
     }
 
+    /// Returns the name of the detected OS version.
+    ///
+    /// *C style function: [notcurses_osversion()][c_api::notcurses_osversion].*
+    pub fn osversion(&self) -> String {
+        rstring_free![c_api::notcurses_detected_terminal(self)]
+    }
+
     /// Destroys all [`NcPlane`]s other than the stdplane.
     ///
     /// *C style function: [notcurses_drop_planes()][c_api::notcurses_drop_planes].*
@@ -355,22 +411,20 @@ impl Nc {
         }
     }
 
-    #[doc(hidden)]
-    #[deprecated = "use `get` method instead"]
-    pub fn getc(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
-        self.get(time, input)
-    }
-
-    /// Returns a [char] representing a single unicode point.
+    /// Gets an [`NcReceived`] from input.
     ///
-    /// If an event is processed, the return value is the `id` field from that
-    /// event.
+    /// Provide `None` in `time` to block at length, and otherwise
+    /// `Some(`[`NcTime`]`)` to bound blocking.
     ///
-    /// Provide a None `time` to block at length, a `time` of 0 for non-blocking
-    /// operation, and otherwise a timespec to bound blocking.
+    /// `time` is an a delay bound against `CLOCK_MONOTONIC`
+    /// (see [*pthread_cond_clockwait(3)*](https://linux.die.net/man/3/pthread_cond_wait)).
     ///
     /// *C style function: [notcurses_get()][c_api::notcurses_get].*
-    pub fn get(&mut self, time: Option<NcTime>, input: Option<&mut NcInput>) -> NcResult<char> {
+    pub fn get(
+        &mut self,
+        time: Option<NcTime>,
+        input: Option<&mut NcInput>,
+    ) -> NcResult<NcReceived> {
         let ntime;
         if let Some(time) = time {
             ntime = &time as *const _;
@@ -386,8 +440,43 @@ impl Nc {
         }
 
         let res = unsafe { c_api::notcurses_get(self, ntime, ninput) };
-        core::char::from_u32(res)
-            .ok_or_else(|| NcError::with_msg(res as i32, &format!["Nc.get(time: {:?})", time]))
+        if res == c_api::NCRESULT_ERR as u32 {
+            Err(NcError::new_msg(&format!["Nc.get({:?})", time]))
+        } else {
+            Ok(NcReceived::new(res))
+        }
+    }
+
+    /// Reads input blocking until an event is processed or a signal is received.
+    ///
+    /// Will optionally write the event details in `input`.
+    ///
+    /// In the case of a valid read, a [`char`] is returned.
+    ///
+    /// *C style function: [notcurses_get_blocking()][c_api::notcurses_get_blocking].*
+    pub fn get_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<NcReceived> {
+        let res = c_api::notcurses_get_blocking(self, input);
+        if res == c_api::NCRESULT_ERR {
+            Err(NcError::new_msg("Nc.get_blocking()"))
+        } else {
+            Ok(NcReceived::new(res as u32))
+        }
+    }
+
+    /// Reads input without blocking.
+    ///
+    /// In the case of a valid read, a [`char`] is returned.
+    ///
+    /// If no event is ready, returns 0.
+    ///
+    /// *C style function: [notcurses_get_nblock()][c_api::notcurses_get_nblock].*
+    pub fn get_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<NcReceived> {
+        let res = c_api::notcurses_get_nblock(self, input);
+        if res == c_api::NCRESULT_ERR {
+            Err(NcError::new_msg("Nc.get_nblock()"))
+        } else {
+            Ok(NcReceived::new(res as u32))
+        }
     }
 
     /// Acquire up to 'vcount' [`NcInput`]s at the vector 'ni'.
@@ -413,34 +502,10 @@ impl Nc {
         error![res, "", res as u32]
     }
 
-    /// Reads input blocking until an event is processed or a signal is received.
-    ///
-    /// Will optionally write the event details in `input`.
-    ///
-    /// In the case of a valid read, a [`char`] is returned.
-    ///
-    /// *C style function: [notcurses_getc_blocking()][c_api::notcurses_getc_blocking].*
-    pub fn getc_blocking(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
-        let res = c_api::notcurses_getc_blocking(self, input);
-        core::char::from_u32(res as u32).ok_or_else(|| NcError::with_msg(res, "Nc.getc_blocking()"))
-    }
-
-    /// Reads input without blocking.
-    ///
-    /// In the case of a valid read, a [`char`] is returned.
-    ///
-    /// If no event is ready, returns 0.
-    ///
-    /// *C style function: [notcurses_getc_nblock()][c_api::notcurses_getc_nblock].*
-    pub fn getc_nblock(&mut self, input: Option<&mut NcInput>) -> NcResult<char> {
-        let res = c_api::notcurses_getc_nblock(self, input);
-        core::char::from_u32(res as u32).ok_or_else(|| NcError::with_msg(res, "Nc.getc_nblock()"))
-    }
-
     /// Gets a file descriptor suitable for input event poll()ing.
     ///
     /// When this descriptor becomes available, you can call
-    /// [getc_nblock()][Nc#method.getc_nblock], and input ought be ready.
+    /// [get_nblock()][Nc#method.get_nblock], and input ought be ready.
     ///
     /// This file descriptor is not necessarily the file descriptor associated
     /// with stdin (but it might be!).
@@ -538,25 +603,24 @@ impl Nc {
         error![unsafe { c_api::notcurses_linesigs_enable(self) }]
     }
 
-    /// Disables mouse events.
+    /// Disables mice events.
     ///
-    /// Any events in the input queue can still be delivered.
-    ///
-    /// *C style function: [notcurses_mouse_disable()][c_api::notcurses_mouse_disable].*
-    pub fn mouse_disable(&mut self) -> NcResult<()> {
-        error![unsafe { c_api::notcurses_mouse_disable(self) }]
+    /// *C style function: [notcurses_mice_disable()][c_api::notcurses_mice_disable].*
+    pub fn mice_disable(&mut self) -> NcResult<()> {
+        self.mice_enable(NcMiceEvents::NO_EVENTS)
     }
 
-    /// Enable the mouse in "button-event tracking" mode with focus detection
-    /// and UTF8-style extended coordinates.
+    /// Enables mice events according to `eventmask`.
     ///
-    /// On success, mouse events will be published to [getc()][Nc#method.getc].
+    /// An eventmask of 0 will disable all mice tracking.
     ///
-    /// *C style function: [notcurses_mouse_enable()][c_api::notcurses_mouse_enable].*
-    pub fn mouse_enable(&mut self) -> NcResult<()> {
+    /// On success mouse events will be published to `notcurses_get`.
+    ///
+    /// *C style function: [notcurses_mice_enable()][c_api::notcurses_mice_enable].*
+    pub fn mice_enable(&mut self, eventmask: NcMiceEvents) -> NcResult<()> {
         error![
-            unsafe { c_api::notcurses_mouse_enable(self) },
-            "Nc.mouse_enable()"
+            unsafe { c_api::notcurses_mice_enable(self, eventmask) },
+            "Nc.mice_enable()"
         ]
     }
 
@@ -600,44 +664,7 @@ impl Nc {
     ///
     /// *C style function: [notcurses_render()][c_api::notcurses_render].*
     pub fn render(&mut self) -> NcResult<()> {
-        error![unsafe { c_api::notcurses_render(self) }, "Nc.render()"]
-    }
-
-    /// Performs the rendering and rasterization portion of
-    /// [`render`][Nc#method.render] but do not write the resulting buffer
-    /// out to the terminal.
-    ///
-    /// Using this function, the user can control the writeout process,
-    /// and render a second frame while writing another.
-    ///
-    // possible BUG? CHECK:
-    /// The returned buffer must be freed by the caller.
-    ///
-    /// *C style function: [notcurses_render_to_buffer()][c_api::notcurses_render_to_buffer].*
-    //
-    // CHECK that this works.
-    #[deprecated]
-    pub fn render_to_buffer(&mut self, buffer: &mut Vec<u8>) -> NcResult<()> {
-        let len = buffer.len() as u32;
-
-        // https://github.com/dankamongmen/notcurses/issues/1339
-        #[cfg(any(target_arch = "x86_64", target_arch = "i686", target_arch = "x86"))]
-        let mut buf = buffer.as_mut_ptr() as *mut i8;
-        #[cfg(not(any(target_arch = "x86_64", target_arch = "i686", target_arch = "x86")))]
-        let mut buf = buffer.as_mut_ptr() as *mut u8;
-
-        error![unsafe { c_api::notcurses_render_to_buffer(self, &mut buf, &mut len.into()) }]
-    }
-
-    /// Writes the last rendered frame, in its entirety, to 'fp'.
-    ///
-    /// If [`render`][Nc#method.render] has not yet been called,
-    /// nothing will be written.
-    ///
-    /// *C style function: [notcurses_render_to_file()][c_api::notcurses_render_to_file].*
-    #[deprecated]
-    pub fn render_to_file(&mut self, fp: &mut NcFile) -> NcResult<()> {
-        error![unsafe { c_api::notcurses_render_to_file(self, fp.as_nc_ptr()) }]
+        error![c_api::notcurses_render(self), "Nc.render()"]
     }
 
     /// Acquires an atomic snapshot of the notcurses object's stats.
@@ -701,9 +728,14 @@ impl Nc {
     /// The standard plane always exists, and its origin is always at the
     /// uppermost, leftmost cell.
     ///
+    /// # Safety
+    /// You must be careful not to end up with multiple exclusive references
+    /// to the standard plane, or with one exclusive reference and one or more
+    /// shared references.
+    ///
     /// *C style function: [notcurses_stdplane()][c_api::notcurses_stdplane].*
-    pub fn stdplane<'a>(&mut self) -> &'a mut NcPlane {
-        unsafe { &mut *c_api::notcurses_stdplane(self) }
+    pub unsafe fn stdplane<'a>(&mut self) -> &'a mut NcPlane {
+        &mut *c_api::notcurses_stdplane(self)
     }
 
     /// Returns a reference to the standard [`NcPlane`] for this terminal.
@@ -711,16 +743,13 @@ impl Nc {
     /// The standard plane always exists, and its origin is always at the
     /// uppermost, leftmost cell.
     ///
-    /// *C style function: [notcurses_stdplane_const()][c_api::notcurses_stdplane_const].*
-    pub fn stdplane_const<'a>(&self) -> &'a NcPlane {
-        unsafe { &*c_api::notcurses_stdplane_const(self) }
-    }
-
-    /// Destroys the notcurses context.
+    /// # Safety
+    /// You must be careful not to end up with a mix of exclusive references
+    /// and shared references to the standard plane.
     ///
-    /// *C style function: [notcurses_stop()][c_api::notcurses_stop].*
-    pub fn stop(&mut self) -> NcResult<()> {
-        error![unsafe { c_api::notcurses_stop(self) }]
+    /// *C style function: [notcurses_stdplane_const()][c_api::notcurses_stdplane_const].*
+    pub unsafe fn stdplane_const<'a>(&self) -> &'a NcPlane {
+        &*c_api::notcurses_stdplane_const(self)
     }
 
     /// Gets the name of an [`NcBlitter`] blitter.
@@ -778,11 +807,19 @@ impl Nc {
         c_api::notcurses_term_dim_yx(self)
     }
 
+    /// Returns the bottommost [`NcPlane`] on the standard pile,
+    /// of which there is always at least one.
+    ///
+    /// *C style function: [notcurses_bottom()][c_api::notcurses_bottom].*
+    pub fn bottom(&mut self) -> &mut NcPlane {
+        c_api::notcurses_bottom(self)
+    }
+
     /// Returns the topmost [`NcPlane`], of which there is always at least one.
     ///
     /// *C style function: [notcurses_top()][c_api::notcurses_top].*
     pub fn top(&mut self) -> &mut NcPlane {
-        unsafe { &mut *c_api::notcurses_top(self) }
+        c_api::notcurses_top(self)
     }
 
     /// Returns a human-readable string describing the running notcurses version.
@@ -803,4 +840,145 @@ impl Nc {
         }
         (major as u32, minor as u32, patch as u32, tweak as u32)
     }
+
+    /// Returns [`NcVisualGeometry`].
+    ///
+    /// If an [`NcVisual`] is not provided, only the [`cdim_yx`], [`blitter`],
+    /// [`scale_yx`], and [`maxpixel_yx`] fields will be filled in.
+    ///
+    /// If an [`NcVisualOptions`] is not provided, a default one will be used.
+    ///
+    /// Additionally `cdim_yx` and `maxpixel_yx` are only ever filled in if we
+    /// know them, and `maxpixel_yx` is only defined for `NcBlitter`::PIXEL.
+    ///
+    /// # See also
+    /// - [`NcVisual.geom`][NcVisual#method.geom]
+    ///
+    /// [`cdim_yx`]: NcVisualGeometry#structfield.cdim_yx
+    /// [`blitter`]: NcVisualGeometry#structfield.blitter
+    /// [`scale_yx`]: NcVisualGeometry#structfield.scale_yx
+    /// [`maxpixel_yx`]: NcVisualGeometry#structfield.maxpixel_yx
+    ///
+    /// *C style function: [ncvisual_geom()][c_api::ncvisual_geom].*
+    pub fn visual_geom(
+        &self,
+        visual: Option<&NcVisual>,
+        vopts: Option<&NcVisualOptions>,
+    ) -> NcResult<NcVisualGeometry> {
+        let mut vg = NcVGeom::new();
+
+        let v_ptr: *const NcVisual;
+        if let Some(v) = visual {
+            v_ptr = v;
+        } else {
+            v_ptr = null();
+        }
+        let vo_ptr: *const NcVisualOptions;
+        if let Some(o) = vopts {
+            vo_ptr = o;
+        } else {
+            let vo = NcVisualOptions::default();
+            vo_ptr = &vo;
+        }
+
+        let res = unsafe { crate::c_api::ncvisual_geom(self, v_ptr, vo_ptr, &mut vg) };
+        if res <= c_api::NCRESULT_ERR {
+            return Err(NcError::with_msg(
+                res,
+                &format!["Nc.visual_geom({:?}, {:?})", visual, vopts],
+            ));
+        }
+
+        let (pix_yx, cdim_yx, rpix_yx, rcell_yx, scale_yx, maxpixel_yx, beg_yx, len_yx);
+
+        // if an `NcVisual` is not provided, only `maxpixel_yx`, `cdim_yx` and
+        // `scale_yx` can be non-zero.
+        if visual.is_none() {
+            cdim_yx = Some((vg.cdimy as NcDim, vg.cdimx as NcDim));
+            scale_yx = Some((vg.scaley as NcDim, vg.scalex as NcDim));
+
+            // pixel blitter only is defined for Ncblitter::PIXEL
+            if vg.blitter == NcBlitter::PIXEL {
+                maxpixel_yx = Some((vg.maxpixely as NcDim, vg.maxpixelx as NcDim));
+            } else {
+                maxpixel_yx = None;
+            }
+
+            pix_yx = None;
+            rpix_yx = None;
+            rcell_yx = None;
+            beg_yx = None;
+            len_yx = None;
+        } else {
+            // `maxpixel_yx` only is defined for `Ncblitter`::PIXEL.
+            if vg.blitter == NcBlitter::PIXEL {
+                maxpixel_yx = Some((vg.maxpixely as NcDim, vg.maxpixelx as NcDim));
+            } else {
+                maxpixel_yx = None;
+            }
+
+            // `beg_yx` & `len_yx` can be safely ignored if they're all 0.
+            if vg.begy | vg.begx | vg.leny | vg.lenx == 0 {
+                beg_yx = None;
+                len_yx = None;
+            } else {
+                beg_yx = Some((vg.begy as NcDim, vg.begx as NcDim));
+                len_yx = Some((vg.leny as NcDim, vg.lenx as NcDim));
+            }
+
+            // valid values for the following fields can't be 0 either:
+            if vg.pixy | vg.pixx == 0 {
+                pix_yx = None;
+            } else {
+                pix_yx = Some((vg.pixy as NcDim, vg.pixx as NcDim));
+            }
+            if vg.cdimy | vg.cdimx == 0 {
+                cdim_yx = None;
+            } else {
+                cdim_yx = Some((vg.cdimy as NcDim, vg.cdimx as NcDim));
+            }
+            if vg.scaley | vg.scalex == 0 {
+                scale_yx = None;
+            } else {
+                scale_yx = Some((vg.scaley as NcDim, vg.scalex as NcDim));
+            }
+            if vg.rpixy | vg.rpixx == 0 {
+                rpix_yx = None;
+            } else {
+                rpix_yx = Some((vg.rpixy as NcDim, vg.rpixx as NcDim));
+            }
+            if vg.rcelly | vg.rcellx == 0 {
+                rcell_yx = None;
+            } else {
+                rcell_yx = Some((vg.rcelly as NcDim, vg.rcellx as NcDim));
+            }
+        }
+
+        let vgeometry = NcVisualGeometry {
+            pix_yx,
+            cdim_yx,
+            rpix_yx,
+            rcell_yx,
+            scale_yx,
+            maxpixel_yx,
+            beg_yx,
+            len_yx,
+
+            blitter: vg.blitter,
+        };
+        Ok(vgeometry)
+    }
+
+    /// Like [`visual_geom`] but auto-fills the `NcVisualOptions` with
+    /// `NcBlitter::PIXEL` in order to get the maximum available resolution
+    /// for `scale_yx`, which determines the minimum dot-size for an `NcVisual`.
+    ///
+    /// [`visual_geom`]: Nc#method.visual_geom
+    pub fn visual_geom_with_pixel(&self, visual: Option<&NcVisual>) -> NcResult<NcVisualGeometry> {
+        Self::visual_geom(
+            self,
+            visual,
+            Some(&NcVisualOptions::builder().pixel().build()),
+        )
+    }
 }
