@@ -4,124 +4,11 @@ use core::ptr::{null, null_mut};
 use libc::c_void;
 
 use crate::{
-    c_api, cstring, error, error_ref_mut, rstring_free, Nc, NcBlitter, NcBlitterApi,
-    NcBlitterGeometry, NcChannel, NcComponent, NcDim, NcDirect, NcError, NcIntResult,
-    NcIntResultApi, NcPixel, NcPlane, NcResult, NcRgba, NcScale, NcTime, NcVGeom, NcVisual,
-    NcVisualOptions,
+    c_api, cstring, error, error_ref_mut, Nc, NcBlitter, NcBlitterApi, NcChannel, NcComponent,
+    NcDim, NcDirect, NcError, NcIntResult, NcIntResultApi, NcPixel, NcPlane, NcResult, NcRgba,
+    NcScale, NcTime, NcVGeom, NcVisual, NcVisualGeometry, NcVisualOptions,
 };
 
-/// # NcVisualOptions Constructors
-impl NcVisualOptions {
-    // /// New NcVisualOptions
-    // pub fn new() -> Self {
-    //     Self::with_flags()
-    // }
-    //
-    // pub fn new_aligned() -> Self {
-    //     Self::with_flags_aligned()
-    // }
-
-    // TODO:
-    // - horizontally aligned
-    // - copy from NcPlaneOptions (with_flags_aligned & with_flags,)
-    // y is an ncalign_e if NCVISUAL_OPTION_VERALIGNED is provided.
-    // x is an ncalign_e value if NCVISUAL_OPTION_HORALIGNED is provided.
-
-    /// Specify an existing plane.
-    ///
-    /// If [`NcVisualOptions::CHILDPLANE`][NcVisualOptions#associatedconstant.CHILDPLANE]
-    /// is used in `flags` then the `plane` is interpreted as the parent
-    /// [`NcPlane`] of the new plane created for this [`NcVisual`].
-    pub fn with_plane(
-        plane: &mut NcPlane,
-        scale: NcScale,
-        y: NcDim,
-        x: NcDim,
-        beg_y: NcDim,
-        beg_x: NcDim,
-        len_y: NcDim,
-        len_x: NcDim,
-        // pxoff_y: NcDim,
-        // pxoff_x: NcDim,
-        blitter: NcBlitter,
-        flags: u32,
-        transcolor: NcRgba,
-    ) -> Self {
-        Self {
-            // provided plane
-            n: plane,
-            // the source is stretched/scaled relative to the provided ncplane
-            scaling: scale,
-            y: y as i32,
-            x: x as i32,
-            // origin of rendered section
-            begy: beg_y as i32,
-            begx: beg_x as i32,
-            // size of rendered section
-            leny: len_y as i32,
-            lenx: len_x as i32,
-            // glyph set to use
-            blitter,
-            // bitmask over NCVISUAL_OPTION_*
-            flags: flags as u64,
-            transcolor,
-            // WIP
-            pxoffy: 0,
-            pxoffx: 0,
-        }
-    }
-
-    // TODO: use Option<> groups for coords
-    pub fn without_plane(
-        y: NcDim,
-        x: NcDim,
-        beg_y: NcDim,
-        beg_x: NcDim,
-        len_y: NcDim,
-        len_x: NcDim,
-        // pxoff_y: NcDim,
-        // pxoff_x: NcDim,
-        blitter: NcBlitter,
-        flags: u32,
-        transcolor: u32,
-    ) -> Self {
-        Self {
-            n: null_mut(),
-            scaling: crate::c_api::NCSCALE_NONE,
-            // where the created plane will be placed relative to stdplane's origin
-            y: y as i32,
-            x: x as i32,
-            // origin of rendered section
-            begy: beg_y as i32,
-            begx: beg_x as i32,
-            // size of rendered section
-            leny: len_y as i32,
-            lenx: len_x as i32,
-            // glyph set to use
-            blitter,
-            // bitmask over NCVISUAL_OPTION_*
-            flags: flags as u64,
-            // This color will be treated as transparent with flag [NCVISUAL_OPTION_ADDALPHA].
-            transcolor,
-            // pixel offsets within the cell.
-            // if NCBLIT_PIXEL is used, the bitmap will be drawn offset from the
-            // upper-left cell's origin by these amounts. it is an error if
-            // either number exceeds the cell-pixel geometry in its dimension.
-            // if NCBLIT_PIXEL is not used, these fields are ignored.
-            // this functionality can be used for smooth bitmap movement.
-            // WIP
-            pxoffy: 0,
-            pxoffx: 0,
-            // pxoffy: pxoff_y,
-            // pxoffx: pxoff_x,
-        }
-    }
-
-    pub fn fullsize_pixel_without_plane(y: NcDim, x: NcDim, len_y: NcDim, len_x: NcDim) -> Self {
-        Self::without_plane(y, x, 0, 0, len_y, len_x, NcBlitter::PIXEL, 0, 0)
-    }
-}
-
 /// # NcVisual Constructors & destructors
 impl NcVisual {
     /// Like [from_rgba][NcVisual#method.from_rgba], but 'bgra' is arranged as BGRA.
@@ -149,7 +36,7 @@ impl NcVisual {
         ]
     }
 
-    /// Opens a visual at `file`, extracts the codec and parameters and
+    /// Opens an `NcVisual` at `file`, extracts the codec and parameters and
     /// decodes the first image to memory.
     ///
     /// *C style function: [ncvisual_from_file()][c_api::ncvisual_from_file].*
@@ -160,37 +47,45 @@ impl NcVisual {
         ]
     }
 
-    /// Promotes an NcPlane to an NcVisual.
+    /// Promotes an `NcPlane` to an `NcVisual`.
     ///
     /// The plane may contain only spaces, half blocks, and full blocks.
     /// This will be checked, and any other glyph will result in an error.
     ///
-    /// This function exists so that planes can be subjected to NcVisual transformations.
-    /// If possible, it's better to create the ncvisual from memory using
-    /// [from_rgba][NcVisual#method.from_rgba].
+    /// This function exists so that planes can be subjected to `NcVisual`
+    /// transformations.
+    ///
+    /// If possible, it's better to create the `NcVisual` from memory using
+    /// [`from_rgba`][NcVisual#method.from_rgba].
+    ///
+    /// Use `None` for either or both of `beg_y` and `beg_x` in order to
+    /// use the current cursor position along that axis.
+    ///
+    /// Use `None` for either or both of `len_y` and `len_x` in order to
+    /// go through the boundary of the plane in that axis (same as `0`).
     ///
     /// *C style function: [ncvisual_from_plane()][c_api::ncvisual_from_plane].*
     pub fn from_plane<'a>(
         plane: &NcPlane,
         blitter: NcBlitter,
-        beg_y: NcDim,
-        beg_x: NcDim,
-        len_y: NcDim,
-        len_x: NcDim,
+        beg_y: Option<NcDim>,
+        beg_x: Option<NcDim>,
+        len_y: Option<NcDim>,
+        len_x: Option<NcDim>,
     ) -> NcResult<&'a mut NcVisual> {
         error_ref_mut![
             unsafe {
                 c_api::ncvisual_from_plane(
                     plane,
                     blitter,
-                    beg_y as i32,
-                    beg_x as i32,
-                    len_y as i32,
-                    len_x as i32,
+                    beg_y.unwrap_or(NcDim::MAX) as i32,
+                    beg_x.unwrap_or(NcDim::MAX) as i32,
+                    len_y.unwrap_or(0),
+                    len_x.unwrap_or(0),
                 )
             },
             &format!(
-                "NcVisual::from_file(plane, {}, {}, {}, {}, {})",
+                "NcVisual::from_file(plane, {}, {:?}, {:?}, {:?}, {:?})",
                 blitter, beg_y, beg_x, len_y, len_x
             )
         ]
@@ -254,12 +149,12 @@ impl NcVisual {
         ]
     }
 
-    /// Prepares an NcVisual, and its underlying NcPlane, based off RGBA content
-    /// in memory at `rgba`.
+    /// Prepares an `NcVisual`, and its underlying `NcPlane`,
+    /// based off RGBA content in memory at `rgba`.
     ///
-    /// `rgba` is laid out as `rows` lines, each of which is `rowstride` bytes in length.
-    /// Each line has `cols` 32-bit 8bpc RGBA pixels followed by possible padding
-    /// (there will be rowstride - cols * 4 bytes of padding).
+    /// `rgba` is laid out as `rows` lines, each of which is `rowstride` bytes
+    /// in length. Each line has `cols` 32-bit 8bpc RGBA pixels followed by
+    /// possible padding (there will be rowstride - cols * 4 bytes of padding).
     ///
     /// The total size of `rgba` is thus (rows * rowstride) bytes, of which
     /// (rows * cols * 4) bytes are actual non-padding data.
@@ -348,15 +243,6 @@ impl NcVisual {
 
 /// # NcVisual Methods
 impl NcVisual {
-    /// Gets the specified pixel from this NcVisual.
-    ///
-    /// *C style function: [ncvisual_at_yx()][c_api::ncvisual_at_yx].*
-    pub fn at_yx(&self, y: NcDim, x: NcDim) -> NcResult<NcPixel> {
-        let mut pixel = 0;
-        let res = unsafe { c_api::ncvisual_at_yx(self, y as i32, x as i32, &mut pixel) };
-        error![res, "NcVisual.at_yx()", pixel]
-    }
-
     /// Extracts the next frame from the NcVisual.
     ///
     /// Returns 0 for normal frames, and 1 to indicate EOF.
@@ -389,58 +275,149 @@ impl NcVisual {
         }
     }
 
-    /// Gets the size and ratio of NcVisual pixels to output cells along the
-    /// vertical and horizontal axes.
+    /// Returns [`NcVisualGeometry`].
     ///
-    /// Returns [`NcBlitterGeometry`].
+    /// if [`Nc`] is not provided, only [`pix_yx`] will be filled in, with the
+    /// true pixel geometry of the current `NcVisual`.
     ///
-    /// An NcVisual of `y` by `x` pixels will require
-    /// (`y` * `to_y`) by (`x` * `to_x`) cells for full output.
+    /// Additionally [`cdim_yx`] and [`maxpixel_yx`] are only ever filled in if we
+    /// know them, and `maxpixel_yx` is only defined for `NcBlitter`::PIXEL.
     ///
-    /// Errors on invalid blitter in `options`. Scaling is taken into consideration.
+    /// # See also
+    /// - [`Nc.visual_geom`][Nc#method.visual_geom]
     ///
-    /// *C style function: [ncvisual_blitter_geom()][c_api::ncvisual_blitter_geom].*
-    pub fn blitter_geom(&self, nc: &Nc, options: &NcVisualOptions) -> NcResult<NcBlitterGeometry> {
-        let mut y = 0;
-        let mut x = 0;
-        let mut scale_y = 0;
-        let mut scale_x = 0;
-        let mut blitter = 0;
+    /// [`pix_yx`]: NcVisualGeometry#structfield.pix_yx
+    /// [`cdim_yx`]: NcVisualGeometry#structfield.cdim_yx
+    /// [`scale_yx`]: NcVisualGeometry#structfield.scale_yx
+    /// [`maxpixel_yx`]: NcVisualGeometry#structfield.maxpixel_yx
+    /// [`blitter`]: NcVisualGeometry#structfield.blitter
+    ///
+    /// *C style function: [ncvisual_geom()][c_api::ncvisual_geom].*
+    pub fn geom(
+        &self,
+        nc: Option<&Nc>,
+        vopts: Option<&NcVisualOptions>,
+    ) -> NcResult<NcVisualGeometry> {
+        let mut vg = NcVGeom::new();
 
-        let res = unsafe {
-            c_api::ncvisual_blitter_geom(
-                nc,
-                self,
-                options,
-                &mut y,
-                &mut x,
-                &mut scale_y,
-                &mut scale_x,
-                &mut blitter,
-            )
+        let nc_ptr: *const Nc;
+        if let Some(nc) = nc {
+            nc_ptr = nc;
+        } else {
+            nc_ptr = null();
+        }
+        let vo_ptr: *const NcVisualOptions;
+        if let Some(o) = vopts {
+            vo_ptr = o;
+        } else {
+            let vo = NcVisualOptions::default();
+            vo_ptr = &vo;
+        }
+
+        let res = unsafe { crate::c_api::ncvisual_geom(nc_ptr, self, vo_ptr, &mut vg) };
+        if res <= c_api::NCRESULT_ERR {
+            return Err(NcError::with_msg(
+                res,
+                &format!["NcVisual.geom({:?}, {:?})", vopts, nc],
+            ));
+        }
+
+        let (pix_yx, cdim_yx, rpix_yx, rcell_yx, scale_yx, maxpixel_yx, beg_yx, len_yx);
+
+        // if an `Nc` context is not provided, only `pix_yx` will be filled in.
+        if nc.is_none() {
+            pix_yx = Some((vg.pixy as NcDim, vg.pixx as NcDim));
+
+            cdim_yx = None;
+            rpix_yx = None;
+            rcell_yx = None;
+            scale_yx = None;
+            maxpixel_yx = None;
+            beg_yx = None;
+            len_yx = None;
+        } else {
+            // `maxpixel_yx` only is defined for `Ncblitter::PIXEL`.
+            if vg.blitter == NcBlitter::PIXEL {
+                maxpixel_yx = Some((vg.maxpixely as NcDim, vg.maxpixelx as NcDim));
+            } else {
+                maxpixel_yx = None;
+            }
+
+            // `beg_yx` & `len_yx` can be safely ignored if they're both all 0.
+            if vg.begy | vg.begx | vg.leny | vg.lenx == 0 {
+                beg_yx = None;
+                len_yx = None;
+            } else {
+                beg_yx = Some((vg.begy as NcDim, vg.begx as NcDim));
+                len_yx = Some((vg.leny as NcDim, vg.lenx as NcDim));
+            }
+
+            // valid values for the following fields can't be 0 either:
+            if vg.pixy | vg.pixx == 0 {
+                pix_yx = None;
+            } else {
+                pix_yx = Some((vg.pixy as NcDim, vg.pixx as NcDim));
+            }
+            if vg.cdimy | vg.cdimx == 0 {
+                cdim_yx = None;
+            } else {
+                cdim_yx = Some((vg.cdimy as NcDim, vg.cdimx as NcDim));
+            }
+            if vg.scaley | vg.scalex == 0 {
+                scale_yx = None;
+            } else {
+                scale_yx = Some((vg.scaley as NcDim, vg.scalex as NcDim));
+            }
+            if vg.rpixy | vg.rpixx == 0 {
+                rpix_yx = None;
+            } else {
+                rpix_yx = Some((vg.rpixy as NcDim, vg.rpixx as NcDim));
+            }
+            if vg.rcelly | vg.rcellx == 0 {
+                rcell_yx = None;
+            } else {
+                rcell_yx = Some((vg.rcelly as NcDim, vg.rcellx as NcDim));
+            }
+        }
+
+        let vgeometry = NcVisualGeometry {
+            pix_yx,
+            cdim_yx,
+            rpix_yx,
+            rcell_yx,
+            scale_yx,
+            maxpixel_yx,
+            beg_yx,
+            len_yx,
+
+            blitter: vg.blitter,
         };
-        let geom = NcBlitterGeometry {
-            x: x as NcDim,
-            y: y as NcDim,
-            scale_y: scale_y as NcDim,
-            scale_x: scale_x as NcDim,
-            blitter,
-        };
-        error![res, "NcVisual.geom()", geom];
+        Ok(vgeometry)
     }
 
     /// Gets the default media (not plot) blitter for this environment when using
     /// the specified scaling method.
     ///
     /// Currently, this means:
-    /// - if lacking UTF-8, `NcBlitter::BLIT_1x1`.
-    /// - otherwise, if not [`NcScale::STRETCH`][NcScale#associatedconstant.STRETCH], `BLIT_2x1`.
-    /// - otherwise, if sextants are not known to be good, `NcBlitter::BLIT_2x2`.
-    /// - otherwise `NCBLIT_3x2`, `NCBLIT_2x2` and `NcBlitter::BLIT_3x2` both
-    ///   distort the original aspect ratio, thus `NCBLIT_2x1` is used
-    ///   outside of [`NcScale::STRETCH`][NcScale#associatedconstant.STRETCH].
+    /// - if lacking UTF-8, [`NcBlitter::ASCII`].
+    /// - otherwise, if not using *[`NcScale::STRETCH`]* then [`NcBlitter::HALF`].
+    /// - otherwise, if sextants are not known to be good, [`NcBlitter::QUADRANT`].
+    /// - otherwise [`NcBlitter::SEXTANT`]
+    ///
+    /// [`QUADRANT`] and [`SEXTANT`] both distort the original aspect ratio,
+    /// thus they are only used alongside *[`NcScale::STRETCH`]*, while [`HALF`]
+    /// is used otherwise.
     ///
     /// *C style function: [ncvisual_media_defblitter()][c_api::ncvisual_media_defblitter].*
+    ///
+    /// [`NcBlitter::ASCII`]: NcBlitter#associatedconstant.ASCII
+    /// [`NcScale::STRETCH`]: NcScale#associatedconstant.STRETCH
+    /// [`NcBlitter::HALF`]: NcBlitter#associatedconstant.HALF
+    /// [`NcBlitter::QUADRANT`]: NcBlitter#associatedconstant.QUADRANT
+    /// [`NcBlitter::SEXTANT`]: NcBlitter#associatedconstant.SEXTANT
+    /// [`HALF`]: NcBlitter#associatedconstant.HALF
+    /// [`QUADRANT`]: NcBlitter#associatedconstant.QUADRANT
+    /// [`SEXTANT`]: NcBlitter#associatedconstant.SEXTANT
     pub fn media_defblitter(nc: &Nc, scale: NcScale) -> NcBlitter {
         unsafe { c_api::ncvisual_media_defblitter(nc, scale) }
     }
@@ -450,27 +427,11 @@ impl NcVisual {
     /// *C style function: [ncvisual_polyfill_yx()][c_api::ncvisual_polyfill_yx].*
     pub fn polyfill_yx(&mut self, y: NcDim, x: NcDim, rgba: NcRgba) -> NcResult<()> {
         error![
-            unsafe { c_api::ncvisual_polyfill_yx(self, y as i32, x as i32, rgba) },
+            unsafe { c_api::ncvisual_polyfill_yx(self, y, x, rgba) },
             &format!["NcVisual.polyfill_yx({}, {}, {})", y, x, rgba]
         ]
     }
 
-    /// Renders the decoded frame to the [`NcPlane`] specified in `options`.
-    ///
-    /// If a plane is not provided, it will be created, having the exact size
-    /// necessary to display the visual.
-    ///
-    /// See [`NcVisualOptions`].
-    ///
-    /// *C style function: [ncvisual_render()][c_api::ncvisual_render].*
-    #[deprecated]
-    pub fn render(&mut self, nc: &mut Nc, options: &NcVisualOptions) -> NcResult<&mut NcPlane> {
-        error_ref_mut![
-            unsafe { c_api::ncvisual_render(nc, self, options) },
-            "NcVisual.render(Nc, &NcVisualOptions)"
-        ]
-    }
-
     /// Renders the decoded frame according to the provided `options`.
     ///
     /// There are 3 options for choosing the the plane used for rendering:
@@ -490,17 +451,19 @@ impl NcVisual {
     ///
     /// Pixels may not be blitted to the standard plane.
     ///
+    /// # Safety
+    /// You must be careful not to end up with multiple exclusive references
+    /// to the returned `NcPlane`, or with one exclusive reference
+    /// and one or more shared references.
+    ///
     /// *C style function: [ncvisual_blit()][c_api::ncvisual_blit].*
-    pub fn blit(
+    pub unsafe fn blit(
         &mut self,
         nc: &mut Nc,
         options: Option<&NcVisualOptions>,
     ) -> NcResult<&mut NcPlane> {
         let options_ptr = if let Some(o) = options { o } else { null() };
-        error_ref_mut![
-            unsafe { c_api::ncvisual_blit(nc, self, options_ptr) },
-            "NcVisual.blit"
-        ]
+        error_ref_mut![c_api::ncvisual_blit(nc, self, options_ptr), "NcVisual.blit"]
     }
 
     /// Resizes the visual to `cols` X `rows` pixels.
@@ -542,12 +505,21 @@ impl NcVisual {
         ]
     }
 
+    /// Gets the specified pixel from this NcVisual.
+    ///
+    /// *C style function: [ncvisual_at_yx()][c_api::ncvisual_at_yx].*
+    pub fn at_yx(&self, y: NcDim, x: NcDim) -> NcResult<NcPixel> {
+        let mut pixel = 0;
+        let res = unsafe { c_api::ncvisual_at_yx(self, y, x, &mut pixel) };
+        error![res, "NcVisual.at_yx()", pixel]
+    }
+
     /// Sets the specified pixel.
     ///
     /// *C style function: [ncvisual_set_yx()][c_api::ncvisual_set_yx].*
     pub fn set_yx(&mut self, y: NcDim, x: NcDim, pixel: NcPixel) -> NcResult<()> {
         error![
-            unsafe { c_api::ncvisual_set_yx(self, y as i32, x as i32, pixel) },
+            unsafe { c_api::ncvisual_set_yx(self, y, x, pixel) },
             &format!["NcVisual.set_yx({}, {}, {})", y, x, pixel]
         ]
     }
@@ -602,8 +574,8 @@ impl NcVisual {
     // /// abort from within streamer().
     // ///
     // /// `timescale` allows the frame duration time to be scaled.
-    // /// For a visual naturally running at 30FPS, a 'timescale' of 0.1 will result
-    // /// in 300 FPS, and a `timescale` of 10 will result in 3 FPS.
+    // /// For an NcVisual naturally running at 30FPS, a 'timescale' of 0.1
+    // /// will result in 300 FPS, and a `timescale` of 10 will result in 3 FPS.
     // /// It is an error to supply `timescale` less than or equal to 0.
     // ///
     // /// *C style function: [ncvisual_streamer()][c_api::ncvisual_streamer].*
@@ -621,20 +593,6 @@ impl NcVisual {
     // ) -> NcResult<()> {
     // }
 
-    /// If a subtitle ought be displayed at this time, returns a heap-allocated
-    /// copy of the UTF8 text.
-    ///
-    /// *C style function: [ncvisual_subtitle()][c_api::ncvisual_subtitle].*
-    #[deprecated]
-    pub fn subtitle(&self) -> NcResult<String> {
-        let res = unsafe { c_api::ncvisual_subtitle(self) };
-        if !res.is_null() {
-            Ok(rstring_free![res])
-        } else {
-            Err(NcError::with_msg(NcIntResult::ERR, "NcVisual.subtitle()"))
-        }
-    }
-
     /// If a subtitle ought be displayed at this time, return a new plane
     ///
     /// The returned plane is bound to `parent` and contains the subtitle,
@@ -685,7 +643,7 @@ impl NcVisual {
     ///
     /// [0]: NcDirect#method.render_frame
     ///
-    /// *C style function: [ncvisual_render()][c_api::ncvisual_render].*
+    /// *C style function: [ncdirectf_render()][c_api::ncdirectf_render].*
     pub fn ncdirectf_render(
         &mut self,
         ncd: &mut NcDirect,
@@ -710,25 +668,3 @@ impl NcVisual {
         error![res, "NcVisual.ncdirectf_geom()", geom];
     }
 }
-
-/// # NcVGeom Constructors
-impl NcVGeom {
-    /// Returns a new `NcVGeom` with zeroed fields.
-    pub fn new() -> Self {
-        Self {
-            pixy: 0,
-            pixx: 0,
-            cdimy: 0,
-            cdimx: 0,
-            rpixy: 0,
-            rpixx: 0,
-            rcelly: 0,
-            rcellx: 0,
-            scaley: 0,
-            scalex: 0,
-            maxpixely: 0,
-            maxpixelx: 0,
-            blitter: 0,
-        }
-    }
-}
